/*============ Insurance Example Data ============*/
data autolosses(keep=policyholderId age gender carType annualMiles
                education carSafety income deductible limit noloss lossamount);
  call streaminit(12345);
  array cx{5} age gender carType annualMiles education;
  array cbeta{6} _TEMPORARY_ (1 -0.75 1 0.6 -1 -0.25);
  array sx{3} carType carSafety income;
  array sbeta{4} _TEMPORARY_ (3.5 1.5 -0.8 0.6);
  alpha = 1/3; theta = 1/alpha; Sigma = 1;
  
  do policyholderId=1 to 5000;
     /* simulate policyholder and vehicle attributes */
     age = MAX(int(rand('NORMAL', 35, 15)),16)/50;
     if (rand('UNIFORM') < 0.5) then gender = 1; * female;
     else gender = 2; * male;
     if (rand('UNIFORM') < 0.7) then carType = 1; * sedan;
     else carType = 2; * SUV;
     annualMiles = MAX(1000, int(rand('NORMAL', 12000, 5000)))/5000;
     educationLevel = rand('UNIFORM');
     if (educationLevel < 0.5) then education = 1; *high school graduate;
     else if (educationLevel < 0.85) then education = 2; *college graduate;
     else education = 3; *advanced degree;
     carSafety = rand('UNIFORM'); /* scaled to be between 0 & 1 */
     income = MAX(15000,int(rand('NORMAL', education*30000, 50000)))/100000;
     deductible = 100*(1+floor(rand('UNIFORM')*5));
     limit = 2500*(1+floor(rand('UNIFORM')*3));
     /* simulate number of losses incurred by this policyholder */
     cxbeta = cbeta(1);
     do i=1 to dim(cx);
        cxbeta = cxbeta + cx(i) * cbeta(i+1);
     end;
     Mu = exp(cxbeta); p = theta/(Mu+theta);
     numloss = rand('NEGB',p,theta);
     /* simulate severity of each of the losses */
     if (numloss > 0) then do;
        noloss = 0;
        do iloss=1 to numloss;
           Mu = sbeta(1);
           do i=1 to dim(sx);
              Mu = Mu + sx(i) * sbeta(i+1);
           end;
           lossamount = deductible + exp(Mu) * rand('LOGNORMAL')**Sigma;
           output;
        end;
     end;
     else do;
        noloss = 1; lossamount = .; deductible = .; limit = .;
        output;
     end;
  end;
run;

data autolosscounts(keep=age gender carType annualMiles education numloss);
  set autolosses;
  by policyholderId;
  retain numloss 0;
  if (noloss ne 1) then numloss = numloss + 1;
  if (last.policyholderId) then do;
     output;
     numloss = 0;
  end;
run;

/* Generate the scenario data set for multiple policyholders */
data groupOfPolicies(keep=policyholderId age gender carType annualMiles education carSafety 
                         income lowDeductible highDeductible limit annualLimit);
  call streaminit(67897);
  do policyholderId=1 to 15;
     age = MAX(int(rand('NORMAL', 35, 15)),16)/50;
     if (rand('UNIFORM') < 0.5) then gender = 1; * female;
     else gender = 2; * male;
     if (rand('UNIFORM') < 0.7) then carType = 1; * sedan;
     else carType = 2; * SUV;
     annualMiles = MAX(1000, int(rand('NORMAL', 12000, 5000)))/5000;
     educationLevel = rand('UNIFORM');
     if (educationLevel < 0.5) then education = 1; *high school graduate;
     else if (educationLevel < 0.85) then education = 2; *college graduate;
     else education = 3; *advanced degree;
     carSafety = rand('UNIFORM'); /* scaled to be between 0 & 1 */
     income = MAX(15000,int(rand('NORMAL', education*30000, 50000)))/100000;
     lowDeductible = 100*(1+floor(rand('UNIFORM')*5));
     highDeductible = lowDeductible + 500*(1+floor(rand('UNIFORM')*2));
     limit = 2500*(1+floor(rand('UNIFORM')*3));
     annualLimit = 10000*(1+floor(rand('UNIFORM')*2));
     output;
  end;
run;

/*============ Operational Risk Example Data ============*/
data OpRiskLossCounts(keep=line corpKRI1 corpKRI2
                    cbKRI1 cbKRI2 cbKRI3 rbKRI1 rbKRI2 numloss);
  call streaminit(12345);
  array cx{7} corpKRI1 corpKRI2 cbKRI1 cbKRI2 cbKRI3 rbKRI1 rbKRI2;
  array cbetaR{8} _TEMPORARY_ (0.1 1 0 0 0 0 0.5 0.25);
  array cbetaC{8} _TEMPORARY_ (0.5 0.75 0.3 0.1 0.25 0.5 0 0);
  alpha = 0.3; theta = 1/alpha;
  do obs=1 to 180;
     do i=1 to dim(cx);
        cx(i) = rand('NORMAL');
     end;
     line = 'CommercialBanking';
     xbeta = cbetaC(1);
     do i=1 to dim(cx);
        xbeta = xbeta + cx(i) * cbetaC(i+1);
     end;
     Mu = exp(xbeta); p = theta/(Mu+theta);
     numloss = rand('NEGB',p,theta);
     output;
     line = 'RetailBanking';
     xbeta = cbetaR(1);
     do i=1 to dim(cx);
        xbeta = xbeta + cx(i) * cbetaR(i+1);
     end;
     lambda = exp(xbeta);
     numloss = rand('POISSON', lambda);
     output;
  end;
run;
proc sort data=OpRiskLossCounts;
  by line;
run;

/* Simulate severity data */
data OpRiskLosses(keep=line corpKRI1 corpKRI2 cbKRI2 rbKRI1 rbKRI3 lossValue);
  array sx{5} corpKRI1 corpKRI2 cbKRI2 rbKRI1 rbKRI3;
  array sbetaC{6} _TEMPORARY_ (5 1 0 0.3 0 0);
  array sbetaR{6} _TEMPORARY_ (3.5 0 0.5 0 -0.8 0.6);
  if (_n_=1) then call streaminit(67890);
  set OpRiskLossCounts(keep=line numloss corpKRI1 corpKRI2 cbKRI2 rbKRI1);
  sigma = 1; alpha = 2.5;
  if (numloss > 0) then do;
     sx(5) = rand('NORMAL'); /* simulate rbKRI3 value */
     if (line='CommercialBanking') then do;
        /* lognormal */
        Mu = sbetaC(1);
        do i=1 to dim(sx);
           Mu = Mu + sx(i) * sbetaC(i+1);
        end;
        lossValue = exp(Mu) * rand('LOGNORMAL')**Sigma;
     end;
     else do;
        /* gamma */
        Mu = sbetaR(1);
        do i=1 to dim(sx);
           Mu = Mu + sx(i) * sbetaR(i+1);
        end;
        lossValue = quantile('Gamma', rand('UNIFORM'), Alpha, exp(Mu));
     end;
     output;
  end;
run;

/* Generate a scenario data set for multiple operating conditions */
data multiConditionScenario (keep=opEnvId corpKRI1 corpKRI2
     cbKRI1 cbKRI2 cbKRI3 rbKRI1 rbKRI2 rbKRI3);
  array x{8} corpKRI1 corpKRI2 cbKRI1 cbKRI2 cbKRI3 rbKRI1 rbKRI2 rbKRI3;
  call streaminit(5151);
  do opEnvId=1 to 12;
     do i=1 to dim(x);
        x(i) = rand('NORMAL');
     end;
     output;
  end;
run;

/*============ Simple PROC HPCDM Example Data ============*/
/* Simulate data for an intercept-only Poisson count model */
data claimcount(keep=numLosses);
  call streaminit(12345);
  label numLosses='Number of Loss Events in a Year';
  Lambda = 2;
  do n = 1 to 500;
     numLosses = rand('POISSON',Lambda);
     output;
  end;
run;
    
/* Simulate data for gamma severity model */
data claimsev(keep=lossValue);
  call streaminit(67890);
  label lossValue='Severity of a Loss Event';
  Theta = 1000; Alpha = 2;
  
  do n = 1 to 500;
     lossValue = quantile('Gamma', rand('UNIFORM'), Alpha, Theta);
     output;
  end;
run;
